/*
	yytext2bmp, A text to bitmap converter
    Copyright (C) 2002 Stephane Magnenat <nct@ysagoon.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <vector>
#include "SDL.h"

typedef struct
{
	int ascii;
	int r, g, b;
} configEntry;

typedef struct
{
	int r, g, b;
} color;

void putpixel32lin(SDL_Surface *surface, int pos, int r, int g, int b, int a)
{
	Uint8 *p=(Uint8 *)surface->pixels + pos * 4;
	*p++=(Uint8)r;
	*p++=(Uint8)g;
	*p++=(Uint8)b;
	*p++=(Uint8)a;
}

int main(int argc, char *argv[])
{
	// cmd line argument
	const char *configFile;
	const char *textFile;
	int width;

	// config data
	vector<configEntry> config;

	// image data
	vector<color> text;
	
	// cmd line parsing
	if (argc!=4)
	{
		printf("Usage : %s width configFile textFile\n", argv[0]);
		exit(1);
	}
	else
	{
		width=atoi(argv[1]);
		configFile=argv[2];
		textFile=argv[3];
		if (!width)
		{
			printf("Error, width must be non zero position integer\n");
			exit(2);
		}
	}

	// read config file
	FILE *configFP=fopen(configFile, "rb");
	char configBuffer[256];
	if (!configFP)
	{
		printf("Error, can't open config file\n");
		exit(3);
	}
	while (fgets(configBuffer, 256, configFP))
	{
		configEntry entry;
		sscanf(configBuffer, "%d %d %d %d", &(entry.ascii), &(entry.r), &(entry.g), &(entry.b));
		config.push_back(entry);
	}
	fclose(configFP);

	// read text file
	FILE *textFP=fopen(textFile, "rb");
	int c;
	if (!textFP)
	{
		printf("Error, can't open text file\n");
		exit(4);
	}
	while ((c=fgetc(textFP))!=EOF)
	{
		vector<configEntry>::iterator entryIt;
		for (entryIt=config.begin(); entryIt!=config.end(); ++entryIt)
		{
			if ((*entryIt).ascii==c)
			{
				color newColor;
				newColor.r=(*entryIt).r;
				newColor.g=(*entryIt).b;
				newColor.b=(*entryIt).b;
				text.push_back(newColor);
			}
		}
	}
	fclose(textFP);

	int height=(int)ceil( ((double)text.size())/((double)width));
	
	// create picture
	SDL_Surface *picture=SDL_CreateRGBSurface(SDL_SWSURFACE, width, height, 32, 0xFF, 0xFF00, 0xFF0000, 0xFF000000);
	vector<color>::iterator colorIt;
	int i=0;
	for (colorIt=text.begin(); colorIt!=text.end(); ++colorIt)
	{
		putpixel32lin(picture, i, (*colorIt).r, (*colorIt).g, (*colorIt).b, SDL_ALPHA_OPAQUE);
		i++;
	}
	while (i<width*height)
	{
		i++;
		putpixel32lin(picture, i, 0, 0, 0, SDL_ALPHA_OPAQUE);
	}

	printf("Output is %dx%d (%d symbols in text, %d colors in palette)\n", width, height, text.size(), config.size());
	SDL_SaveBMP(picture, "out.bmp");
	SDL_FreeSurface(picture);

	return 0;
}
